@@ -17,11 +17,13 @@
 This server provides tools for analyzing AWS service costs across different user tiers.
 """
 
+import re
 import sys
 from awslabs.aws_pricing_mcp_server import consts
 from awslabs.aws_pricing_mcp_server.cdk_analyzer import analyze_cdk_project
 from awslabs.aws_pricing_mcp_server.models import (
     ATTRIBUTE_NAMES_FIELD,
+    ATTRIBUTE_VALUES_FILTERS_FIELD,
     EFFECTIVE_DATE_FIELD,
     FILTERS_FIELD,
     GET_PRICING_MAX_ALLOWED_CHARACTERS_FIELD,
@@ -29,10 +31,12 @@ from awslabs.aws_pricing_mcp_server.models import (
     NEXT_TOKEN_FIELD,
     OUTPUT_OPTIONS_FIELD,
     REGION_FIELD,
+    SERVICE_ATTRIBUTES_FILTER_FIELD,
     SERVICE_CODE_FIELD,
+    SERVICE_CODES_FILTER_FIELD,
     ErrorResponse,
     OutputOptions,
-    PricingFilters,
+    PricingFilter,
 )
 from awslabs.aws_pricing_mcp_server.pricing_client import (
     create_pricing_client,
@@ -99,7 +103,7 @@ mcp = FastMCP(
        instance_types = get_pricing_attribute_values('AmazonEC2', 'instanceType')
 
        # Get pricing for specific instance types in a region
-       filters = {"filters": [{"Field": "instanceType", "Value": "t3.medium", "Type": "TERM_MATCH"}]}
+       filters = [{"Field": "instanceType", "Value": "t3.medium", "Type": "TERM_MATCH"}]
        pricing = get_pricing('AmazonEC2', 'us-east-1', filters)
 
        # Get bulk pricing data files for historical analysis
@@ -228,7 +232,7 @@ async def analyze_terraform_project_wrapper(
     **PARAMETERS:**
     - service_code (required): AWS service code (e.g., 'AmazonEC2', 'AmazonS3', 'AmazonES')
     - region (required): AWS region string (e.g., 'us-east-1') OR list for multi-region comparison (e.g., ['us-east-1', 'eu-west-1'])
-    - filters (optional): PricingFilters object with list of filter dictionaries
+    - filters (optional): List of filter dictionaries in format {'Field': str, 'Type': str, 'Value': str}
     - max_allowed_characters (optional): Response size limit in characters (default: 100,000, use -1 for unlimited)
     - output_options (optional): OutputOptions object for response transformation and size reduction
     - max_results (optional): Maximum number of results to return per page (default: 100, min: 1, max: 100)
@@ -246,13 +250,11 @@ async def analyze_terraform_project_wrapper(
     **Step 2: Build Precise Filters**
     ```python
     # Use ONLY values discovered in Step 1
-    filters = {
-       "filters": [
-           {"Field": "memory", "Value": ["8 GiB", "16 GiB", "32 GiB"], "Type": "ANY_OF"},     # Multiple options
-           {"Field": "instanceType", "Value": "m5", "Type": "CONTAINS"},                      # Pattern matching
-           {"Field": "instanceType", "Value": ["t2", "m4"], "Type": "NONE_OF"}                # Exclude older
-       ]
-    }
+    filters = [
+       {"Field": "memory", "Value": ["8 GiB", "16 GiB", "32 GiB"], "Type": "ANY_OF"},     # Multiple options
+       {"Field": "instanceType", "Value": "m5", "Type": "CONTAINS"},                      # Pattern matching
+       {"Field": "instanceType", "Value": ["t2", "m4"], "Type": "NONE_OF"}                # Exclude older
+   ]
     ```
 
     **Step 3: Execute Query**
@@ -266,7 +268,7 @@ async def analyze_terraform_project_wrapper(
     - **CONTAINS**: Pattern match - `{"Field": "instanceType", "Value": "m5", "Type": "CONTAINS"}`
     - **NONE_OF**: Exclusion - `{"Field": "instanceType", "Value": ["t2", "m4"], "Type": "NONE_OF"}`
 
-    **⚠️ CRITICAL: ANY_OF FILTER VALUE LIMITS:**
+    **CRITICAL: ANY_OF FILTER VALUE LIMITS:**
     - **1024 CHARACTER LIMIT**: Total length of all values in ANY_OF arrays cannot exceed 1024 characters
     - **PROGRESSIVE FILTERING**: Start with minimal qualifying options, expand if needed
     - **EXAMPLE VIOLATION**: `["8 GiB", "16 GiB", "32 GiB", "64 GiB", "96 GiB", "128 GiB", ...]` (TOO LONG)
@@ -278,7 +280,7 @@ async def analyze_terraform_project_wrapper(
     - **LOWER = CHEAPER ASSUMPTION**: For cost optimization, assume lower capabilities cost less than higher ones
       * 32 GB storage is cheaper than 300 GB storage
       * 8 GiB RAM is cheaper than 64 GiB RAM
-    - **⚠️ CRITICAL FOR COST QUERIES**: Start IMMEDIATELY above minimum requirement and test ALL options incrementally
+    - **CRITICAL FOR COST QUERIES**: Start IMMEDIATELY above minimum requirement and test ALL options incrementally
     - **EXHAUSTIVE ENUMERATION REQUIRED**: Each storage/memory tier is MUTUALLY EXCLUSIVE - must list each one explicitly
     - **STOP AT REASONABLE UPPER BOUND**: For cost optimization, limit upper bound to 2-3x minimum requirement to avoid expensive options
     - **exclude_free_products**: ESSENTIAL for cost analysis - removes $0.00 reservation placeholders, SQL licensing variants, and special pricing entries that obscure actual billable instances when finding cheapest options
@@ -308,16 +310,16 @@ async def analyze_terraform_project_wrapper(
     - **CHARACTER LIMIT**: 100,000 characters default response limit (use output_options to reduce)
     - **REGION AUTO-FILTER**: Region parameter automatically creates regionCode filter
 
-    **ANTI-PATTERNS - AVOID THESE:**
-    ❌ Making multiple API calls that could be combined with ANY_OF
-    ❌ Building cross-products manually when API can handle combinations
-    ❌ Calling get_pricing_service_codes() when service code is already known (e.g., "AmazonEC2")
-    ❌ Using EQUALS without first checking get_pricing_attribute_values()
-    ❌ Skipping discovery workflow for any use case
-    ❌ Using broad queries without specific filters on large services
-    ❌ Assuming attribute values exist across different services/regions
-    ❌ **SKIPPING INTERMEDIATE TIERS**: Missing 50GB, 59GB options when testing 32GB → 75GB jump
-    ❌ **SETTING UPPER BOUNDS TOO HIGH**: Including 500GB+ storage when user needs ≥30GB (wastes character limit)
+    **ANTI-PATTERNS:**
+    - DO NOT make multiple API calls that could be combined with ANY_OF
+    - DO NOT build cross-products manually when API can handle combinations
+    - DO NOT call get_pricing_service_codes() when service code is already known (e.g., "AmazonEC2")
+    - DO NOT use EQUALS without first checking get_pricing_attribute_values()
+    - DO NOT skip discovery workflow for any use case
+    - DO NOT use broad queries without specific filters on large services
+    - DO NOT assume attribute values exist across different services/regions
+    - DO NOT skip intermediate tiers: Missing 50GB, 59GB options when testing 32GB → 75GB jump
+    - DO NOT set upper bounds too high: Including 500GB+ storage when user needs ≥30GB (wastes character limit)
 
     **EXAMPLE USE CASES:**
 
@@ -325,19 +327,19 @@ async def analyze_terraform_project_wrapper(
     ```python
     # Find cheapest EC2 instances meeting minimum requirements (>= 8 GiB memory, >= 30 GB storage)
     # EXHAUSTIVE ENUMERATION of qualifying tiers - each is mutually exclusive
-    filters = {"filters": [
+    filters = [
        {"Field": "memory", "Value": ["8 GiB", "16 GiB", "32 GiB"], "Type": "ANY_OF"},  # All tiers ≥8GB up to reasonable limit
        {"Field": "storage", "Value": ["1 x 32 SSD", "1 x 60 SSD", "1 x 75 NVMe SSD"], "Type": "ANY_OF"},  # All tiers ≥30GB up to reasonable limit
        {"Field": "instanceType", "Value": ["t2", "m4"], "Type": "NONE_OF"},  # Exclude older generations
        {"Field": "tenancy", "Value": "Shared", "Type": "EQUALS"}  # Exclude more expensive dedicated
-    ]}
+    ]
     pricing = get_pricing('AmazonEC2', 'us-east-1', filters)
     ```
 
     **2. Efficient Multi-Region Comparison:**
     ```python
     # Compare same configuration across regions - use region parameter for multi-region
-    filters = {"filters": [{"Field": "instanceType", "Value": "m5.large", "Type": "EQUALS"}]}
+    filters = [{"Field": "instanceType", "Value": "m5.large", "Type": "EQUALS"}]
     pricing = get_pricing('AmazonEC2', ['us-east-1', 'us-west-2', 'eu-west-1'], filters)
     ```
 
@@ -350,10 +352,10 @@ async def analyze_terraform_project_wrapper(
     **4. Pattern-Based Discovery with Refinement:**
     ```python
     # Find all Standard storage tiers except expensive ones
-    filters = {"filters": [
+    filters = [
         {"Field": "storageClass", "Value": "Standard", "Type": "CONTAINS"},
         {"Field": "storageClass", "Value": ["Standard-IA"], "Type": "NONE_OF"}
-    ]}
+    ]
     ```
 
     **FILTERING STRATEGY:**
@@ -364,18 +366,18 @@ async def analyze_terraform_project_wrapper(
     - **Smart Exclusion**: Use NONE_OF for compliance or cost filtering
 
     **SUCCESS CRITERIA:**
-    ✅ Used discovery workflow (skip get_pricing_service_codes() if service known)
-    ✅ Applied appropriate filters for the service size
-    ✅ Used exact values from get_pricing_attribute_values()
-    ✅ Used ANY_OF for multi-option scenarios instead of multiple calls
-    ✅ For cost optimization: tested ALL qualifying tiers exhaustively (in a reasonable range)
+    - Used discovery workflow (skip get_pricing_service_codes() if service known)
+    - Applied appropriate filters for the service size
+    - Used exact values from get_pricing_attribute_values()
+    - Used ANY_OF for multi-option scenarios instead of multiple calls
+    - For cost optimization: tested ALL qualifying tiers exhaustively (in a reasonable range)
     """,
 )
 async def get_pricing(
     ctx: Context,
     service_code: str = SERVICE_CODE_FIELD,
     region: Union[str, List[str]] = REGION_FIELD,
-    filters: Optional[PricingFilters] = FILTERS_FIELD,
+    filters: Optional[List[PricingFilter]] = FILTERS_FIELD,
     max_allowed_characters: int = GET_PRICING_MAX_ALLOWED_CHARACTERS_FIELD,
     output_options: Optional[OutputOptions] = OUTPUT_OPTIONS_FIELD,
     max_results: int = MAX_RESULTS_FIELD,
@@ -435,8 +437,8 @@ async def get_pricing(
         ]
 
         # Add any additional filters if provided
-        if filters and filters.filters:
-            api_filters.extend([f.model_dump(by_alias=True) for f in filters.filters])
+        if filters:
+            api_filters.extend([f.model_dump(by_alias=True) for f in filters])
 
         # Make the API request
         api_params = {
@@ -465,14 +467,19 @@ async def get_pricing(
         return await create_error_response(
             ctx=ctx,
             error_type='empty_results',
-            message=f'The service "{service_code}" did not return any pricing data. AWS service codes typically follow patterns like "AmazonS3", "AmazonEC2", "AmazonES", etc. Please check the exact service code and try again.',
+            message=f'No results found for given filters [{filters}], service: "{service_code}", region "{region}"',
             service_code=service_code,
             region=region,
+            suggestion='Try these approaches: (1) Verify that the service code is valid. Use get_service_codes() to get valid service codes. (2) Validate region and filter values using get_pricing_attribute_values(). (3) Test with fewer filters to isolate the issue.',
             examples={
-                'OpenSearch': 'AmazonES',
-                'Lambda': 'AWSLambda',
-                'DynamoDB': 'AmazonDynamoDB',
-                'Bedrock': 'AmazonBedrock',
+                'Example service codes': [
+                    'AmazonEC2',
+                    'AmazonS3',
+                    'AmazonES',
+                    'AWSLambda',
+                    'AmazonDynamoDB',
+                ],
+                'Example regions': ['us-east-1', 'eu-west-1', 'ap-south-1'],
             },
         )
 
@@ -806,6 +813,9 @@ async def generate_cost_report_wrapper(
 
     **PURPOSE:** Discover which AWS services have pricing information available in the AWS Price List API.
 
+    **PARAMETERS:**
+    - filter (optional): Case-insensitive regex pattern to filter service codes (e.g., "bedrock" matches "AmazonBedrock", "AmazonBedrockService")
+
     **WORKFLOW:** This is the starting point for any pricing query. Use this first to find the correct service code.
 
     **RETURNS:** List of service codes (e.g., 'AmazonEC2', 'AmazonS3', 'AWSLambda') that can be used with other pricing tools.
@@ -817,11 +827,14 @@ async def generate_cost_report_wrapper(
     **NOTE:** Service codes may differ from AWS console names (e.g., 'AmazonES' for OpenSearch, 'AWSLambda' for Lambda).
     """,
 )
-async def get_pricing_service_codes(ctx: Context) -> Union[List[str], Dict[str, Any]]:
+async def get_pricing_service_codes(
+    ctx: Context, filter: Optional[str] = SERVICE_CODES_FILTER_FIELD
+) -> Union[List[str], Dict[str, Any]]:
     """Retrieve all available service codes from AWS Price List API.
 
     Args:
         ctx: MCP context for logging and state management
+        filter: Optional regex pattern to filter service codes (case-insensitive)
 
     Returns:
         List of sorted service codes on success, or error dictionary on failure
@@ -845,18 +858,14 @@ async def get_pricing_service_codes(ctx: Context) -> Union[List[str], Dict[str,
 
         # Retrieve all service codes with pagination handling
         while True:
-            if next_token:
-                response = pricing_client.describe_services(NextToken=next_token)
-            else:
-                response = pricing_client.describe_services()
+            response = pricing_client.describe_services(
+                **({'NextToken': next_token} if next_token else {})
+            )
+            service_codes.extend([service['ServiceCode'] for service in response['Services']])
 
-            for service in response['Services']:
-                service_codes.append(service['ServiceCode'])
-
-            if 'NextToken' in response:
-                next_token = response['NextToken']
-            else:
+            if 'NextToken' not in response:
                 break
+            next_token = response['NextToken']
 
     except Exception as e:
         return await create_error_response(
@@ -874,10 +883,41 @@ async def get_pricing_service_codes(ctx: Context) -> Union[List[str], Dict[str,
             message='No service codes returned from AWS Price List API',
         )
 
-    sorted_codes = sorted(service_codes)
+    # Apply regex filtering if filter is provided
+    if filter:
+        try:
+            regex_pattern = re.compile(filter, re.IGNORECASE)
+            service_codes = [code for code in service_codes if regex_pattern.search(code)]
 
-    logger.info(f'Successfully retrieved {len(sorted_codes)} service codes')
-    await ctx.info(f'Successfully retrieved {len(sorted_codes)} service codes')
+            if not service_codes:
+                return await create_error_response(
+                    ctx=ctx,
+                    error_type='no_matches_found',
+                    message=f'No service codes match the regex pattern: "{filter}"',
+                    filter=filter,
+                    suggestion='Try a broader regex pattern or check the pattern syntax. Use get_pricing_service_codes() without filter to see all available service codes.',
+                )
+
+        except re.error as e:
+            return await create_error_response(
+                ctx=ctx,
+                error_type='invalid_regex',
+                message=f'Invalid regex pattern "{filter}": {str(e)}',
+                filter=filter,
+                suggestion='Please provide a valid regex pattern. For simple substring matching, just use the text without special regex characters.',
+                examples={
+                    'Simple substring': 'bedrock',
+                    'Case-insensitive exact match': '^AmazonBedrock$',
+                    'Starts with': '^Amazon',
+                    'Contains word': '\\bbedrock\\b',
+                },
+            )
+
+    sorted_codes = sorted(service_codes)
+    filter_msg = f' (filtered with pattern: "{filter}")' if filter else ''
+
+    logger.info(f'Successfully retrieved {len(sorted_codes)} service codes{filter_msg}')
+    await ctx.info(f'Successfully retrieved {len(sorted_codes)} service codes{filter_msg}')
 
     return sorted_codes
 
@@ -890,7 +930,9 @@ async def get_pricing_service_codes(ctx: Context) -> Union[List[str], Dict[str,
 
     **WORKFLOW:** Use this after get_pricing_service_codes() to see what filters you can apply to narrow down pricing queries.
 
-    **REQUIRES:** Service code from get_pricing_service_codes() (e.g., 'AmazonEC2', 'AmazonRDS').
+    **PARAMETERS:**
+    - service_code: AWS service code from get_pricing_service_codes() (e.g., 'AmazonEC2', 'AmazonRDS')
+    - filter (optional): Case-insensitive regex pattern to filter attribute names (e.g., "instance" matches "instanceType", "instanceFamily")
 
     **RETURNS:** List of attribute names (e.g., 'instanceType', 'location', 'storageClass') that can be used as filters.
 
@@ -902,17 +944,24 @@ async def get_pricing_service_codes(ctx: Context) -> Union[List[str], Dict[str,
     """,
 )
 async def get_pricing_service_attributes(
-    ctx: Context, service_code: str = SERVICE_CODE_FIELD
+    ctx: Context,
+    service_code: str = SERVICE_CODE_FIELD,
+    filter: Optional[str] = SERVICE_ATTRIBUTES_FILTER_FIELD,
 ) -> Union[List[str], Dict[str, Any]]:
     """Retrieve all available attributes for a specific AWS service.
 
     Args:
         service_code: The service code to query (e.g., 'AmazonEC2', 'AmazonS3')
+        filter: Optional regex pattern to filter attribute names (case-insensitive)
         ctx: MCP context for logging and state management
 
     Returns:
         List of sorted attribute name strings on success, or error dictionary on failure
     """
+    # Handle Pydantic Field objects when called directly (not through MCP framework)
+    if isinstance(filter, FieldInfo):
+        filter = filter.default
+
     logger.info(f'Retrieving attributes for AWS service: {service_code}')
 
     # Create pricing client with error handling
@@ -970,11 +1019,46 @@ async def get_pricing_service_attributes(
             suggestion='This service may not support attribute-based filtering, or there may be a temporary issue. Try using get_pricing() without filters.',
         )
 
-    sorted_attributes = sorted(attributes)
+    # Apply regex filtering if filter is provided
+    if filter:
+        try:
+            regex_pattern = re.compile(filter, re.IGNORECASE)
+            attributes = [attr for attr in attributes if regex_pattern.search(attr)]
 
-    logger.info(f'Successfully retrieved {len(sorted_attributes)} attributes for {service_code}')
+            if not attributes:
+                return await create_error_response(
+                    ctx=ctx,
+                    error_type='no_matches_found',
+                    message=f'No service attributes match the regex pattern: "{filter}"',
+                    service_code=service_code,
+                    filter=filter,
+                    suggestion='Try a broader regex pattern or check the pattern syntax. Use get_pricing_service_attributes() without filter to see all available service attributes.',
+                )
+
+        except re.error as e:
+            return await create_error_response(
+                ctx=ctx,
+                error_type='invalid_regex',
+                message=f'Invalid regex pattern "{filter}": {str(e)}',
+                service_code=service_code,
+                filter=filter,
+                suggestion='Please provide a valid regex pattern. For simple substring matching, just use the text without special regex characters.',
+                examples={
+                    'Simple substring': 'instance',
+                    'Case-insensitive exact match': '^instanceType$',
+                    'Starts with': '^instance',
+                    'Contains word': '\\binstance\\b',
+                },
+            )
+
+    sorted_attributes = sorted(attributes)
+    filter_msg = f' (filtered with pattern: "{filter}")' if filter else ''
+
+    logger.info(
+        f'Successfully retrieved {len(sorted_attributes)} attributes for {service_code}{filter_msg}'
+    )
     await ctx.info(
-        f'Successfully retrieved {len(sorted_attributes)} attributes for {service_code}'
+        f'Successfully retrieved {len(sorted_attributes)} attributes for {service_code}{filter_msg}'
     )
 
     return sorted_attributes
@@ -1076,11 +1160,12 @@ async def _get_single_attribute_values(
 
     **WORKFLOW:** Use this after get_pricing_service_attributes() to see valid values for each filter attribute.
 
-    **REQUIRES:**
+    **PARAMETERS:**
     - Service code from get_pricing_service_codes() (e.g., 'AmazonEC2', 'AmazonRDS')
     - List of attribute names from get_pricing_service_attributes() (e.g., ['instanceType', 'location'])
+    - filters (optional): Dictionary mapping attribute names to regex patterns (e.g., {'instanceType': 't3'})
 
-    **RETURNS:** Dictionary mapping attribute names to their valid values.
+    **RETURNS:** Dictionary mapping attribute names to their valid values. Filtered attributes return only matching values, unfiltered attributes return all values.
 
     **EXAMPLE RETURN:**
     ```
@@ -1097,23 +1182,29 @@ async def _get_single_attribute_values(
     **EXAMPLES:**
     - Single attribute: ['instanceType'] returns {'instanceType': ['t2.micro', 't3.medium', ...]}
     - Multiple attributes: ['instanceType', 'location'] returns both mappings
+    - Partial filtering: filters={'instanceType': 't3'} applies only to instanceType, location returns all values
     """,
 )
 async def get_pricing_attribute_values(
     ctx: Context,
     service_code: str = SERVICE_CODE_FIELD,
     attribute_names: List[str] = ATTRIBUTE_NAMES_FIELD,
+    filters: Optional[Dict[str, str]] = ATTRIBUTE_VALUES_FILTERS_FIELD,
 ) -> Union[Dict[str, List[str]], Dict[str, Any]]:
     """Retrieve all possible values for specific attributes of an AWS service.
 
     Args:
         service_code: The service code to query (e.g., 'AmazonEC2', 'AmazonS3')
         attribute_names: List of attribute names to get values for (e.g., ['instanceType', 'location'])
+        filters: Optional dictionary mapping attribute names to regex patterns for filtering
         ctx: MCP context for logging and state management
 
     Returns:
         Dictionary mapping attribute names to sorted lists of values on success, or error dictionary on failure
     """
+    if isinstance(filters, FieldInfo):
+        filters = filters.default
+
     if not attribute_names:
         return await create_error_response(
             ctx=ctx,
@@ -1149,7 +1240,42 @@ async def get_pricing_attribute_values(
             values_result = await _get_single_attribute_values(
                 pricing_client, service_code, attribute_name
             )
-            # Success - add to result
+
+            # Apply filtering if a filter is provided for this attribute
+            if filters and attribute_name in filters:
+                filter_pattern = filters[attribute_name]
+                logger.debug(f'Applying filter "{filter_pattern}" to attribute "{attribute_name}"')
+
+                try:
+                    regex_pattern = re.compile(filter_pattern, re.IGNORECASE)
+                    filtered_values = [
+                        value for value in values_result if regex_pattern.search(value)
+                    ]
+
+                    # Use filtered values (even if empty list)
+                    values_result = sorted(filtered_values)
+
+                except re.error as e:
+                    # If regex is invalid, return error for entire operation
+                    return await create_error_response(
+                        ctx=ctx,
+                        error_type='invalid_regex',
+                        message=f'Invalid regex pattern "{filter_pattern}" for attribute "{attribute_name}": {str(e)}',
+                        service_code=service_code,
+                        attribute_name=attribute_name,
+                        filter_pattern=filter_pattern,
+                        requested_attributes=attribute_names,
+                        filters=filters,
+                        suggestion='Please provide a valid regex pattern. For simple substring matching, just use the text without special regex characters.',
+                        examples={
+                            'Simple substring': 't3',
+                            'Case-insensitive exact match': '^t3\\.medium$',
+                            'Starts with': '^t3',
+                            'Contains word': '\\bt3\\b',
+                        },
+                    )
+
+            # Success - add to result (filtered or unfiltered)
             result[attribute_name] = values_result
         except AttributeValuesError as e:
             # If any attribute fails, return error for entire operation
@@ -1183,7 +1309,7 @@ async def get_pricing_attribute_values(
 
     **WORKFLOW:** Use this for historical pricing analysis or bulk data processing when current pricing from get_pricing() isn't sufficient.
 
-    **REQUIRES:**
+    **PARAMETERS:**
     - Service code from get_pricing_service_codes() (e.g., 'AmazonEC2', 'AmazonS3')
     - AWS region (e.g., 'us-east-1', 'eu-west-1')
     - Optional: effective_date for historical pricing (default: current date)
@@ -1196,6 +1322,7 @@ async def get_pricing_attribute_values(
     - Historical pricing analysis (get_pricing() only provides current pricing)
     - Bulk data processing without repeated API calls
     - Offline analysis of complete pricing datasets
+    - Savings Plans analysis across services
 
     **FILE PROCESSING:**
     - CSV files: Lines 1-5 are metadata, Line 6 contains headers, Line 7+ contains pricing data
